@@ -24,6 +24,12 @@
 #if defined (TM_IN_SYS_TIME) && defined (HAVE_SYS_TIME_H)
 # include <sys/time.h>
 #endif /* defined (TM_IN_SYS_TIME) && defined (HAVE_SYS_TIME_H) */
+#if GLIB_CHECK_VERSION(2,6,0)
+# include <glib/gstdio.h>
+#endif /* GLIB_CHECK_VERSION(2,6,0) */
+#ifdef G_OS_WIN32
+# include <shlwapi.h>
+#endif /* G_OS_WIN32 */
 
 
 struct _FileIO
@@ -44,123 +50,6 @@ struct _FileIO
 * ja:ファイル名関数群                                                         *
 *                                                                             *
 ******************************************************************************/
-/*  ja:ファイルの存在確認
-    file,ファイル名
-    test,条件
-     RET,TRUE:合致する,FALSE:合致しない                                     */
-gboolean
-fileio_test (const gchar *file,
-             const guint  test)
-{
-#ifdef G_OS_WIN32
-  HANDLE hFind; /* 検索ハンドル */
-# if ! GLIB_CHECK_VERSION(2,6,0)
-#  ifdef UNICODE
-  gchar *utf8str;
-#  endif /* UNICODE */
-  LPTSTR lpszFile;
-  SHFILEINFO rcInfo;
-  WIN32_FIND_DATA rcFind;
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
-#else /* not G_OS_WIN32 */
-  struct stat buf;
-#endif /* not G_OS_WIN32 */
-
-  if (!file)
-    return FALSE;
-#ifdef G_OS_WIN32
-# if GLIB_CHECK_VERSION(2,6,0)
-  if (G_WIN32_HAVE_WIDECHAR_API ())
-    {
-      gchar *utf8str;
-      LPWSTR lpszFile;
-      SHFILEINFOW rcInfo;
-      WIN32_FIND_DATAW rcFind;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
-      utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-      if (!utf8str)
-        return FALSE;
-      lpszFile = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
-      g_free (utf8str);
-      if (!lpszFile)
-        return FALSE;
-      if ((test & FILEIO_TEST_IS_EXECUTABLE) && SHGetFileInfoW (lpszFile, 0,
-                                &rcInfo, sizeof (SHFILEINFOW), SHGFI_EXETYPE))
-        {
-          g_free (lpszFile);
-          return TRUE;
-        }
-      hFind = FindFirstFileW (lpszFile, &rcFind);
-      g_free (lpszFile);
-      if (hFind != INVALID_HANDLE_VALUE)
-        {
-          FindClose (hFind);
-          if ((test & FILEIO_TEST_EXISTS)
-            || ((test & FILEIO_TEST_IS_REGULAR)
-                    && !(rcFind.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
-            || ((test & FILEIO_TEST_IS_DIR)
-                    && (rcFind.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)))
-            return TRUE;
-        }
-# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-    }
-  else
-    {
-      LPSTR lpszFile;
-      SHFILEINFOA rcInfo;
-      WIN32_FIND_DATAA rcFind;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
-      lpszFile = g_win32_locale_filename_from_utf8 (file);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-      lpszFile = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-      lpszFile = g_strdup (file);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
-      if (!lpszFile)
-        return FALSE;
-      if ((test & FILEIO_TEST_IS_EXECUTABLE) && SHGetFileInfoA (lpszFile, 0,
-                                &rcInfo, sizeof (SHFILEINFOA), SHGFI_EXETYPE))
-        {
-          g_free (lpszFile);
-          return TRUE;
-        }
-      hFind = FindFirstFileA (lpszFile, &rcFind);
-      g_free (lpszFile);
-      if (hFind != INVALID_HANDLE_VALUE)
-        {
-          FindClose (hFind);
-          if ((test & FILEIO_TEST_EXISTS)
-            || ((test & FILEIO_TEST_IS_REGULAR)
-                    && !(rcFind.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
-            || ((test & FILEIO_TEST_IS_DIR)
-                    && (rcFind.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)))
-            return TRUE;
-        }
-# endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-    }
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-  return FALSE;
-#else /* not G_OS_WIN32 */
-  if (((test & FILEIO_TEST_EXISTS) && !access (file, F_OK))
-        || ((test & FILEIO_TEST_IS_EXECUTABLE)
-                                        && !access (file, X_OK) && getuid ()))
-    return TRUE;
-  if (lstat (file, &buf))
-    return FALSE;
-  return ((test & FILEIO_TEST_IS_SYMLINK) && S_ISLNK (buf.st_mode))
-                || ((test & FILEIO_TEST_IS_REGULAR) && S_ISREG (buf.st_mode))
-                || ((test & FILEIO_TEST_IS_DIR) && S_ISDIR (buf.st_mode));
-#endif /* not G_OS_WIN32 */
-}
-
-
 /*  ja:ファイルの絶対パスを取得する
     file,ファイル名
     path,パス名
@@ -172,11 +61,11 @@ fileio_get_absolute_path (const gchar *file,
 #ifdef G_OS_WIN32
   gchar *utf8str, *filename;
   gint i = 0, j, s;
-  DWORD dwSize;             /* フルパスの長さ */
-  HANDLE hFind;             /* 検索ハンドル */
+  DWORD dwSize;             /* ja:フルパスの長さ */
+  HANDLE hFind;             /* ja:検索ハンドル */
 # if ! GLIB_CHECK_VERSION(2,6,0)
   LPTSTR p, lpszFile;
-  TCHAR szFile[MAX_PATH];   /* フルパス */
+  TCHAR szFile[MAX_PATH];   /* ja:フルパス */
   TCHAR lpszLongName[MAX_PATH];
 # endif /* not GLIB_CHECK_VERSION(2,6,0) */
 
@@ -186,7 +75,7 @@ fileio_get_absolute_path (const gchar *file,
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
       LPWSTR p, lpszFile;
-      WCHAR szFile[MAX_PATH];   /* フルパス */
+      WCHAR szFile[MAX_PATH];   /* ja:フルパス */
       WCHAR szLongName[MAX_PATH];
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
@@ -194,8 +83,41 @@ fileio_get_absolute_path (const gchar *file,
       utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
       if (!utf8str)
         return g_strdup (file);
-      lpszFile = g_utf8_to_utf16 (file, -1, NULL, NULL, NULL);
+      lpszFile = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
       g_free (utf8str);
+      if (PathIsRelativeW (lpszFile))
+        {
+          gint leng;
+          LPWSTR lpszPath = NULL, lpszTemp;
+
+          if (path)
+            {
+              utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
+              if (utf8str)
+                {
+                  lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
+                  g_free (utf8str);
+                }
+            }
+          if (!lpszPath || PathIsRelativeW (lpszPath))
+            {
+              g_free (lpszPath);
+              lpszPath = g_malloc (MAX_PATH * sizeof (WCHAR));
+              GetCurrentDirectoryW (MAX_PATH, lpszPath);
+            }
+          leng = lstrlenW (lpszPath);
+          if (leng > 0 && G_IS_DIR_SEPARATOR (lpszPath[leng - 1]))
+            lpszPath[--leng] = '\0';
+          lpszTemp = g_malloc ((leng + lstrlenW (lpszFile) + 2)
+                                                            * sizeof (WCHAR));
+          lstrcpyW (lpszTemp, lpszPath);
+          lpszTemp[leng] = G_DIR_SEPARATOR;
+          lpszTemp[leng + 1] = '\0';
+          lstrcatW (lpszTemp, lpszFile);
+          g_free (lpszPath);
+          g_free (lpszFile);
+          lpszFile = lpszTemp;
+        }
       dwSize = GetFullPathNameW (lpszFile, MAX_PATH, szFile, &p);
       g_free (lpszFile);
       if (dwSize == 0 || dwSize >= MAX_PATH)
@@ -270,7 +192,7 @@ fileio_get_absolute_path (const gchar *file,
   else
     {
       LPSTR p, lpszFile;
-      CHAR szFile[MAX_PATH];   /* フルパス */
+      CHAR szFile[MAX_PATH];   /* ja:フルパス */
       CHAR szLongName[MAX_PATH];
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
@@ -282,6 +204,31 @@ fileio_get_absolute_path (const gchar *file,
 #  else /* not GLIB_CHECK_VERSION(2,6,0) */
       lpszFile = g_strdup (file);
 #  endif /* not GLIB_CHECK_VERSION(2,6,0) */
+      if (PathIsRelativeA (lpszFile))
+        {
+          LPSTR lpszPath = NULL, lpszTemp;
+
+          if (path)
+            {
+#  if GLIB_CHECK_VERSION(2,8,0)
+              lpszPath = g_win32_locale_filename_from_utf8 (path);
+#  elif GLIB_CHECK_VERSION(2,6,0)
+              lpszPath = g_locale_from_utf8 (path, -1, NULL, NULL, NULL);
+#  else /* not GLIB_CHECK_VERSION(2,6,0) */
+              lpszPath = g_strdup (path);
+#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
+            }
+          if (!lpszPath || PathIsRelativeA (lpszPath))
+            {
+              g_free (lpszPath);
+              lpszPath = g_malloc (MAX_PATH * sizeof (CHAR));
+              GetCurrentDirectoryA (MAX_PATH, lpszPath);
+            }
+          lpszTemp = g_build_filename (lpszPath, lpszFile, NULL);
+          g_free (lpszPath);
+          g_free (lpszFile);
+          lpszFile = lpszTemp;
+        }
       dwSize = GetFullPathNameA (lpszFile, MAX_PATH, szFile, &p);
       g_free (lpszFile);
       if (dwSize == 0 || dwSize >= MAX_PATH)
@@ -353,19 +300,23 @@ fileio_get_absolute_path (const gchar *file,
 # endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
 # if GLIB_CHECK_VERSION(2,6,0)
     }
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
   if (!utf8str)
     return g_strdup (file);
   filename = g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL);
   g_free (utf8str);
   return filename ? filename : g_strdup (file);
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
 #else /* not G_OS_WIN32 */
   gchar *absolate;
   gint i, leng;
 
   if (!file)
     return NULL;
-  if (!G_IS_DIR_SEPARATOR (file[0]))
+  if (g_path_is_absolute (file))
+    {
+      absolate = g_strdup (file);
+    }
+  else
     {
       if (path && g_path_is_absolute (path))
         {
@@ -383,10 +334,6 @@ fileio_get_absolute_path (const gchar *file,
           g_free (dir);
         }
     }
-  else
-    {
-      absolate = g_strdup (file);
-    }
   leng = g_strlen (absolate);
   i = 0;
   while (i < leng)
@@ -403,10 +350,10 @@ fileio_get_absolute_path (const gchar *file,
   i = 0;
   while (i < leng)
     if (G_IS_DIR_SEPARATOR (absolate[i]) && absolate[i + 1] == '.'
-                                    && G_IS_DIR_SEPARATOR (absolate[i + 2]))
+        && (G_IS_DIR_SEPARATOR (absolate[i + 2]) || absolate[i + 2] == '\0'))
       {
         leng -= 2;
-        g_memmove (absolate + i + 1, absolate + i + 3, leng - i);
+        g_memmove (absolate + i, absolate + i + 2, leng - i + 1);
       }
     else
       {
@@ -415,18 +362,19 @@ fileio_get_absolute_path (const gchar *file,
   i = 0;
   while (i < leng)
     if (G_IS_DIR_SEPARATOR (absolate[i]) && absolate[i + 1] == '.'
-            && absolate[i + 2] == '.' && G_IS_DIR_SEPARATOR (absolate[i + 3]))
+        && absolate[i + 2] == '.'
+        && (G_IS_DIR_SEPARATOR (absolate[i + 3])  || absolate[i + 3] == '\0'))
       {
         gint j;
 
         leng -= 3;
-        g_memmove (absolate + i + 1, absolate + i + 4, leng - i);
+        g_memmove (absolate + i, absolate + i + 3, leng - i + 1);
         for (j = i - 1; j >= 0; j--)
           if (G_IS_DIR_SEPARATOR (absolate[j]))
             break;
         if (j >= 0)
           {
-            g_memmove (absolate + j + 1, absolate + i + 1, leng - i);
+            g_memmove (absolate + j, absolate + i, leng - i + 1);
             leng -= i - j;
             i = j;
           }
@@ -435,6 +383,11 @@ fileio_get_absolute_path (const gchar *file,
       {
         i++;
       }
+  for (i = 0; i < leng; i++)
+    if (G_IS_DIR_SEPARATOR (absolate[i]))
+      absolate[i] = G_DIR_SEPARATOR;
+  if (leng <= 0)
+    g_strcpy (absolate, G_DIR_SEPARATOR_S);
   return absolate;
 #endif /* not G_OS_WIN32 */
 }
@@ -448,7 +401,7 @@ gchar *
 fileio_get_relative_path (const gchar *file,
                           const gchar *path)
 {
-  gchar *absolate, *relative = NULL, *name, *tmp;
+  gchar *absolate, *relative = NULL, *name;
   gchar *path_to, *path_from, **ary_to, **ary_from;
   gint i, j;
 
@@ -485,6 +438,8 @@ fileio_get_relative_path (const gchar *file,
   for (j = i; ary_from[j]; j++)
     if (*(ary_from[j]) != '\0')
       {
+        gchar *tmp;
+
         tmp = relative ? g_strconcat (relative, ".."G_DIR_SEPARATOR_S, NULL)
                        : g_strdup (".."G_DIR_SEPARATOR_S);
         g_free (relative);
@@ -492,6 +447,8 @@ fileio_get_relative_path (const gchar *file,
       }
   for (j = i; ary_to[j]; j++)
     {
+      gchar *tmp;
+
       tmp = relative
                 ? g_strconcat (relative, ary_to[j], G_DIR_SEPARATOR_S, NULL)
                 : g_strconcat (ary_to[j], G_DIR_SEPARATOR_S, NULL);
@@ -503,6 +460,8 @@ fileio_get_relative_path (const gchar *file,
   name = g_path_get_basename (file);
   if (name)
     {
+      gchar *tmp;
+
       tmp = relative ? g_strconcat (relative, name, NULL) : g_strdup (name);
       g_free (relative);
       relative = tmp;
@@ -565,7 +524,7 @@ fileio_open (const gchar *file,
       gchar *dir;
 
       dir = g_path_get_dirname (path);
-      if (!fileio_isfile (dir) && !fileio_mkdir (dir))
+      if (!g_file_test (dir, G_FILE_TEST_EXISTS) && !fileio_mkdir (dir))
         {
           g_free (path);
           g_free (dir);
@@ -1116,7 +1075,7 @@ fileio_mkdir (const gchar *dir)
       g_free (parent);
       return TRUE;
     }
-  result = !fileio_isfile (parent) && !fileio_mkdir (parent);
+  result = !g_file_test (parent, G_FILE_TEST_EXISTS) && !fileio_mkdir (parent);
   g_free (parent);
   if (result)
     {
@@ -1404,16 +1363,21 @@ fileio_getsystime (FileIOTime    *ftime,
 ******************************************************************************/
 /*  ja:ファイルのアイコンを取得する
          file,ファイル名
-    icon_size,FILEIO_ICON_SIZE_SMALL/FILEIO_ICON_SIZE_LARGE
+    icon_size,アイコンサイズ
           RET,pixbuf,NULL:エラー                                            */
 GdkPixbuf *
 fileio_extract_icon (const gchar *file,
-                     const guint  icon_size)
+                     GtkIconSize  icon_size)
 {
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32)
+  gint icon_x, icon_y;
   GdkPixbuf *pixbuf = NULL;
   HICON hIcon = NULL;
   ICONINFO ii;
+  UINT uFlags;
+# if GLIB_CHECK_VERSION(2,2,0)
+  GtkSettings *settings;
+# endif /* GLIB_CHECK_VERSION(2,2,0) */
 # if ! GLIB_CHECK_VERSION(2,6,0)
   LPTSTR lpszFile;
   SHFILEINFO shfi;
@@ -1424,6 +1388,34 @@ fileio_extract_icon (const gchar *file,
 
   if (!file)
     return NULL;
+  switch (icon_size)
+    {
+      case GTK_ICON_SIZE_MENU:          icon_x = 16; icon_y = 16; break;
+      case GTK_ICON_SIZE_SMALL_TOOLBAR: icon_x = 18; icon_y = 18; break;
+      case GTK_ICON_SIZE_LARGE_TOOLBAR: icon_x = 24; icon_y = 24; break;
+      case GTK_ICON_SIZE_BUTTON:        icon_x = 20; icon_y = 20; break;
+      case GTK_ICON_SIZE_DND:           icon_x = 32; icon_y = 32; break;
+      case GTK_ICON_SIZE_DIALOG:        icon_x = 48; icon_y = 48; break;
+      default:                          icon_x =  0; icon_y =  0;
+    }
+# if GLIB_CHECK_VERSION(2,2,0)
+  settings = gtk_settings_get_default ();
+  if (settings)
+    {
+      gint w, h;
+
+      if (gtk_icon_size_lookup_for_settings (settings, icon_size, &w, &h))
+        {
+          icon_x = w;
+          icon_y = h;
+        }
+    }
+  uFlags = ABS (GetSystemMetrics (SM_CXSMICON) - icon_x)
+                                + ABS (GetSystemMetrics (SM_CYSMICON) - icon_y)
+                                < ABS (GetSystemMetrics (SM_CXICON) - icon_x)
+                                + ABS (GetSystemMetrics (SM_CYSMICON) - icon_y)
+                                        ? SHGFI_SMALLICON : SHGFI_LARGEICON;
+# endif /* GLIB_CHECK_VERSION(2,2,0) */
 # if GLIB_CHECK_VERSION(2,6,0)
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
@@ -1438,9 +1430,7 @@ fileio_extract_icon (const gchar *file,
         return NULL;
       lpszFile = g_utf8_to_utf16 (file, -1, NULL, NULL, NULL);
       if ((BOOL)SHGetFileInfoW (lpszFile, 0, &shfi, sizeof (SHFILEINFOW),
-                                        SHGFI_ICON | SHGFI_USEFILEATTRIBUTES
-                                        | (icon_size == FILEIO_ICON_SIZE_SMALL
-                                        ? SHGFI_SMALLICON : SHGFI_LARGEICON)))
+                                SHGFI_ICON | SHGFI_USEFILEATTRIBUTES | uFlags))
         hIcon = shfi.hIcon;
       g_free (lpszFile);
 # endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
@@ -1461,13 +1451,13 @@ fileio_extract_icon (const gchar *file,
       lpszFile = g_strdup (file);
 #  endif /* not GLIB_CHECK_VERSION(2,8,0) */
       if ((BOOL)SHGetFileInfoA (lpszFile, 0, &shfi, sizeof (SHFILEINFOW),
-                                        SHGFI_ICON | SHGFI_USEFILEATTRIBUTES
-                                        | (icon_size == FILEIO_ICON_SIZE_SMALL
-                                        ? SHGFI_SMALLICON : SHGFI_LARGEICON)))
+                                SHGFI_ICON | SHGFI_USEFILEATTRIBUTES | uFlags))
         hIcon = shfi.hIcon;
       g_free (lpszFile);
 # endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
+# if GLIB_CHECK_VERSION(2,6,0)
     }
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
   if (GetIconInfo (hIcon, &ii))
     {
       BITMAPINFOHEADER bmih;
@@ -1532,8 +1522,165 @@ fileio_extract_icon (const gchar *file,
       DeleteObject (ii.hbmColor);
       DeleteObject (ii.hbmMask);
     }
+  if (hIcon)
+    DestroyIcon (hIcon);
   return pixbuf;
-#else /* not G_OS_WIN32 */
+#elif GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE)
+  GdkPixbuf *pixbuf = NULL;
+
+  if (!file)
+    return NULL;
+  if (g_file_test (file, G_FILE_TEST_IS_DIR))
+    {
+      const gchar *home;
+      gint leng;
+      GtkWidget *image;
+
+      home = g_get_home_dir ();
+      leng = g_strlen (home);
+      image = gtk_image_new ();
+      pixbuf = gtk_widget_render_icon (image,
+            g_strfilencmp (file, home, leng) == 0
+            && (file[leng] == '\0'
+                || (G_IS_DIR_SEPARATOR (file[leng]) && file[leng + 1] == '\0'))
+                    ? GTK_STOCK_HOME : GTK_STOCK_DIRECTORY, icon_size, NULL);
+      gtk_widget_destroy (image);
+    }
+  else
+    {
+      gchar *text;
+      gssize length;
+
+      text = fileio_load (MIMEFILE, &length);
+      if (text)
+        {
+          gchar **mimes, *utf8str, *mime_type = NULL;
+          gint i;
+
+          text = g_realloc (text, length + 1);
+          text[length] = '\0';
+          utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+          mimes = g_strsplit (text, "\n", 0);
+          for (i = 0; mimes[i]; i++)
+            {
+              gchar **globs;
+
+              globs = g_strsplit (mimes[i], G_SEARCHPATH_SEPARATOR_S, 0);
+              if (globs[0])
+                {
+                  gint j;
+
+                  for (j = 1; globs[j]; j++)
+                    if (g_pattern_match_simple (globs[j], utf8str))
+                      {
+                        mime_type = g_strdup (globs[0]);
+                        break;
+                      }
+                }
+              g_strfreev (globs);
+              if (mime_type)
+                break;
+            }
+          g_strfreev (mimes);
+          g_free (utf8str);
+          g_free (text);
+          if (mime_type)
+            {
+              gchar *icon_name, *separator;
+
+              icon_name = g_strconcat ("gnome-mime-", mime_type, NULL);
+              g_free (mime_type);
+              separator = g_strchr (icon_name, '/');
+              if (separator)
+                {
+                  GtkIconTheme *icon_theme;
+
+                  *separator = '-';
+                  icon_theme = gtk_icon_theme_get_default ();
+                  if (icon_theme)
+                    {
+                      gboolean result;
+
+                      result = gtk_icon_theme_has_icon (icon_theme, icon_name);
+                      if (!result)
+                        {
+                          *separator = '\0';
+                          result = gtk_icon_theme_has_icon (icon_theme,
+                                                                    icon_name);
+                        }
+                      if (result)
+                        {
+                          gint icon_x, icon_y;
+# if GLIB_CHECK_VERSION(2,6,0)
+                          gint *sizes;
+# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+                          GtkSettings *settings;
+
+                          switch (icon_size)
+                            {
+                              case GTK_ICON_SIZE_MENU:
+                                icon_x = icon_y = 16;
+                                break;
+                              case GTK_ICON_SIZE_SMALL_TOOLBAR:
+                                icon_x = icon_y = 18;
+                                break;
+                              case GTK_ICON_SIZE_LARGE_TOOLBAR:
+                                icon_x = icon_y = 24;
+                                break;
+                              case GTK_ICON_SIZE_BUTTON:
+                                icon_x = icon_y = 20;
+                                break;
+                              case GTK_ICON_SIZE_DND:
+                                icon_x = icon_y = 32;
+                                break;
+                              case GTK_ICON_SIZE_DIALOG:
+                                icon_x = icon_y = 48;
+                                break;
+                              default:
+                                icon_x = icon_y = 0;
+                            }
+                          settings = gtk_settings_get_default ();
+                          if (settings)
+                            {
+                              gint w, h;
+
+                              if (gtk_icon_size_lookup_for_settings (settings,
+                                                            icon_size, &w, &h))
+                                {
+                                  icon_x = w;
+                                  icon_y = h;
+                                }
+                            }
+# if GLIB_CHECK_VERSION(2,6,0)
+                          sizes = gtk_icon_theme_get_icon_sizes (icon_theme,
+                                                                    icon_name);
+                          if (sizes)
+                            {
+                              gint n = 0;
+
+                              for (i = 0; sizes[i] > 0; i++)
+                                if (ABS (icon_x - sizes[i])
+                                                    + ABS (icon_y - sizes[i])
+                                        < ABS (icon_x - n) + ABS (icon_y - n))
+                                  n = sizes[i];
+                              g_free (sizes);
+                              if (n > 0)
+                                pixbuf = gtk_icon_theme_load_icon (icon_theme,
+                                                        icon_name, n, 0, NULL);
+                            }
+                          if (!pixbuf)
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+                            pixbuf = gtk_icon_theme_load_icon (icon_theme,
+                                    icon_name, (icon_x + icon_y) / 2, 0, NULL);
+                        }
+                    }
+                }
+              g_free (icon_name);
+            }
+        }
+    }
+  return pixbuf;
+#else /* not G_OS_WIN32 && not MIMEFILE */
   return NULL;
-#endif /* not G_OS_WIN32 */
+#endif /* not G_OS_WIN32 && not MIMEFILE */
 }
