@@ -34,9 +34,9 @@
 #include "misc/fileio.h"
 #include "orz/orzcharlist.h"
 #include "orz/orzcharset.h"
-#include "orz/orzconf.h"
 #include "orz/orzhistory.h"
 #include "orz/orzmdi.h"
+#include "orz/orzpref.h"
 #ifdef G_OS_WIN32
 # include <gdk/gdkwin32.h>
 #endif /* G_OS_WIN32 */
@@ -1369,27 +1369,27 @@ command_all (GtkWidget *widget,
 
 
 void
-command_conf (GtkWidget *widget,
+command_pref (GtkWidget *widget,
               gpointer   user_data)
 {
   GtkWidget *dialog;
 
-  dialog = orz_conf_new ();
+  dialog = orz_pref_new ();
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
-  orz_conf_set_history (ORZ_CONF (dialog),
+  orz_pref_set_history (ORZ_PREF (dialog),
                                 orz_history_get_num (ORZ_HISTORY (history)));
-  orz_conf_set_newfile (ORZ_CONF (dialog), newfile);
-  orz_conf_set_second (ORZ_CONF (dialog), second);
-  orz_conf_set_size (ORZ_CONF (dialog), def_width, def_height);
-  orz_conf_set_tab (ORZ_CONF (dialog), n_pos);
+  orz_pref_set_newfile (ORZ_PREF (dialog), newfile);
+  orz_pref_set_second (ORZ_PREF (dialog), second);
+  orz_pref_set_size (ORZ_PREF (dialog), def_width, def_height);
+  orz_pref_set_tab (ORZ_PREF (dialog), n_pos);
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
     {
       orz_history_set_num (ORZ_HISTORY (history),
-                                    orz_conf_get_history (ORZ_CONF (dialog)));
-      newfile = orz_conf_get_newfile (ORZ_CONF (dialog));
-      second = orz_conf_get_second (ORZ_CONF (dialog));
-      orz_conf_get_size (ORZ_CONF (dialog), &def_width, &def_height);
-      n_pos = orz_conf_get_tab (ORZ_CONF (dialog));
+                                    orz_pref_get_history (ORZ_PREF (dialog)));
+      newfile = orz_pref_get_newfile (ORZ_PREF (dialog));
+      second = orz_pref_get_second (ORZ_PREF (dialog));
+      orz_pref_get_size (ORZ_PREF (dialog), &def_width, &def_height);
+      n_pos = orz_pref_get_tab (ORZ_PREF (dialog));
       /* ja:ノートの位置 */
       gtk_notebook_set_tab_pos (GTK_NOTEBOOK (mdi), n_pos);
     }
