@@ -16,8 +16,8 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
-#include <locale.h>
 #include "mmaid.h"
+#include <locale.h>
 #include "command.h"
 #include "file.h"
 #include "general.h"
@@ -57,9 +57,9 @@ gint playlist = 0;                  /* ja:プレイリスト */
 GObject *history;                   /* ja:ファイルの履歴 */
 GdkAtom atom_targets;               /* ja:ターゲット */
 GdkAtom atom_id3tag;                /* ja:ID3タグ */
+GtkPositionType n_pos = GTK_POS_TOP;/* ja:ノートブックの位置 */
 GtkWidget *window;                  /* ja:ウインドウ */
 GtkWidget *mdi;                     /* en:MDI */
-GtkPositionType n_pos = GTK_POS_TOP;/* ja:ノートブックの位置 */
 #ifdef G_OS_WIN32
 UINT uFormat = 0;                   /* ja:クリップボード */
 #endif /* G_OS_WIN32 */
@@ -104,7 +104,7 @@ MiscCreateMenuEntry menu_entries[] = {
 #endif /* not GTK_CHECK_VERSION(2,10,0) */
 {"/option",             N_("_Option"),             NULL,              NULL,             MISC_CREATE_MENU_ITEM,                              NULL,            NULL, NULL},
 {"/option/",            NULL,                      NULL,              NULL,             MISC_CREATE_MENU_SHELL,                             NULL,            NULL, NULL},
-{"/option/preferences", GTK_STOCK_PREFERENCES,     NULL,              NULL,             MISC_CREATE_MENU_STOCK,                             command_conf,    NULL, NULL},
+{"/option/preferences", GTK_STOCK_PREFERENCES,     NULL,              NULL,             MISC_CREATE_MENU_STOCK,                             command_pref,    NULL, NULL},
 {"/option/genre",       N_("_Genre"),              NULL,              genre16_xpm,      MISC_CREATE_MENU_ITEM | MISC_CREATE_MENU_ELLIPSIS,  command_genre,   NULL, NULL},
 {"/option/character",   N_("Character _Encoding"), NULL,              character16_xpm,  MISC_CREATE_MENU_ITEM | MISC_CREATE_MENU_ELLIPSIS,  command_charset, NULL, NULL},
 {"/option/process",     N_("_Process"),            NULL,              process16_xpm,    MISC_CREATE_MENU_ITEM | MISC_CREATE_MENU_ELLIPSIS,  command_process, NULL, NULL},
@@ -174,19 +174,8 @@ main (int   argc,
       char *argv[])
 {
   gboolean *arg_files;
-  gchar *group_name, *key, *utf8str, *data, *user_config = NULL;
-#if ! GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32)
-  LPITEMIDLIST pidl = NULL;
-#endif /* ! GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32) */
-#ifndef LOCALEDIR
-  gchar *dirname, *path, *locale;
-# if GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32)
-  gchar *temp;
-# endif /* GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32) */
-#endif /* not LOCALEDIR */
-  gsize length;
+  const gchar *user_config = NULL;
   gint i, j, count, license = -1;
-  MmaidWindow *mmaid = NULL;
   GKeyFile *key_file;
   GList *glist;
   GObject *accel, *winmenu;
@@ -201,34 +190,7 @@ main (int   argc,
   setlocale (LC_ALL, "");
   gtk_set_locale ();
   gtk_init (&argc, &argv);
-#ifdef LOCALEDIR
-  bindtextdomain (PACKAGE, LOCALEDIR);
-#else /* not LOCALEDIR */
-  path = fileio_get_full_path (argv[0]);
-  dirname = g_path_get_dirname (path);
-  g_free (path);
-  locale = g_build_filename (dirname, "locale", NULL);
-  g_free (dirname);
-# ifdef G_OS_WIN32
-  if (!g_file_test (locale, G_FILE_TEST_EXISTS))
-    {
-      g_free (locale);
-      locale = g_win32_get_package_installation_subdirectory
-                            (NULL, argv[0], "lib" G_DIR_SEPARATOR_S "locale");
-    }
-#  if GLIB_CHECK_VERSION(2,6,0)
-#   if GLIB_CHECK_VERSION(2,8,0)
-  temp = g_win32_locale_filename_from_utf8 (locale);
-#   else /* not GLIB_CHECK_VERSION(2,8,0) */
-  temp = g_locale_from_utf8 (locale, -1, NULL, NULL, NULL);
-#   endif /* not GLIB_CHECK_VERSION(2,8,0) */
-  g_free (locale);
-  locale = temp;
-#  endif /* GLIB_CHECK_VERSION(2,6,0) */
-# endif /* G_OS_WIN32 */
-  bindtextdomain (PACKAGE, locale);
-  g_free (locale);
-#endif /* not LOCALEDIR */
+  bindtextdomain (PACKAGE, misc_spec_locale (argv[0]));
   bind_textdomain_codeset (PACKAGE, "UTF-8");
   textdomain (PACKAGE);
 
@@ -240,54 +202,22 @@ main (int   argc,
   history = orz_history_new ();
 
   /* ja:キーファイルを求める */
-#ifdef G_OS_WIN32
-# if GLIB_CHECK_VERSION(2,6,0)
-  user_config = g_build_filename (g_get_user_config_dir (),
-                                    "maid.org", "mmaid", "mmaid.ini", NULL);
-# else /* not GLIB_CHECK_VERSION(2,6,0) */
-  if (SHGetSpecialFolderLocation (NULL, CSIDL_APPDATA, &pidl) == S_OK)
-    {
-      TCHAR szPath[MAX_PATH + 1];
-
-      if (SHGetPathFromIDList (pidl, szPath))
-#  ifdef UNICODE
-        {
-          gchar *dir;
-
-          utf8str = g_utf16_to_utf8 (szPath, -1, NULL, NULL, NULL);
-          dir = g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL);
-          g_free (utf8str);
-          user_config = g_build_filename (dir,
-                                    "maid.org", "mmaid", "mmaid.ini", NULL);
-          g_free (dir);
-        }
-#  else /* not UNICODE */
-        user_config = g_build_filename (szPath,
-                                    "maid.org", "mmaid", "mmaid.ini", NULL);
-#  endif /* not UNICODE */
-      CoTaskMemFree (pidl);
-    }
-  if (!user_config)
-    user_config = g_build_filename (g_get_home_dir (), "mmaid.ini", NULL);
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
-#else /* not G_OS_WIN32 */
-  user_config = g_build_filename (g_get_home_dir (), ".mmaid", NULL);
-#endif /* not G_OS_WIN32 */
+  user_config = misc_spec_config ("maid.org", "mmaid", "main");
 
   /* ja:レジストリから読み出す */
   key_file = g_key_file_new ();
   if (g_key_file_load_from_file (key_file, user_config, G_KEY_FILE_NONE, NULL))
     {
-      license = g_key_file_get_integer (key_file, "mmaid", "license", NULL);
+      gchar *utf8str;
+
+      license = g_key_file_get_integer (key_file, "license", "hash", NULL);
       orz_history_set_num (ORZ_HISTORY (history),
-                g_key_file_get_integer (key_file, "mmaid", "history", NULL));
-      newfile = g_key_file_get_boolean (key_file, "mmaid", "newfile", NULL);
-      second = g_key_file_get_boolean (key_file, "mmaid", "second", NULL);
-      def_width = g_key_file_get_integer (key_file, "mmaid", "def_width",
-                                                                        NULL);
-      def_height = g_key_file_get_integer (key_file, "mmaid", "def_height",
-                                                                        NULL);
-      n_pos = g_key_file_get_integer (key_file, "mmaid", "n_pos", NULL);
+                g_key_file_get_integer (key_file, "pref", "history", NULL));
+      newfile = g_key_file_get_boolean (key_file, "pref", "newfile", NULL);
+      second = g_key_file_get_boolean (key_file, "pref", "second", NULL);
+      def_width = g_key_file_get_integer (key_file, "pref", "width", NULL);
+      def_height = g_key_file_get_integer (key_file, "pref", "height", NULL);
+      n_pos = g_key_file_get_integer (key_file, "pref", "n_pos", NULL);
       def_hold = g_key_file_get_boolean (key_file, "mmaid", "def_hold", NULL);
       def_gnum = g_key_file_get_integer (key_file, "mmaid", "def_gnum", NULL);
       playlist = g_key_file_get_integer (key_file, "mmaid", "playlist", NULL);
@@ -314,8 +244,10 @@ main (int   argc,
       /* ja:ファイルの履歴 */
       for (i = orz_history_get_num (ORZ_HISTORY (history)) - 1; i >= 0; i--)
         {
-          key = g_strdup_printf ("history%d", i);
-          utf8str = g_key_file_get_string (key_file, "mmaid", key, NULL);
+          gchar *key;
+
+          key = g_strdup_printf ("file%d", i);
+          utf8str = g_key_file_get_string (key_file, "history", key, NULL);
           g_free (key);
           if (utf8str)
             {
@@ -330,10 +262,14 @@ main (int   argc,
       /* ja:ジャンル */
       for (i = 0; i < 256; i++)
         {
+          gchar *group_name;
+
           group_name = g_strdup_printf ("genre%03d", i);
           count = 0;
           for (j = 0; j < 256; j++)
             {
+              gchar *key;
+
               key = g_strdup_printf ("genre%03d", j);
               genre[i][j] = g_key_file_get_string (key_file, group_name, key,
                                                                         NULL);
@@ -400,7 +336,7 @@ main (int   argc,
   /* ja:コマンドラインの解析 */
   {
     gint array[2], def[2];
-    ArgumentBool arg_n, arg_x;
+    ArgumentBool arg_n, arg_p, arg_x;
     ArgumentValue arg_h;
     ArgumentArray arg_s;
     ArgumentExit arg_v;
@@ -408,6 +344,7 @@ main (int   argc,
     ArgumentList arg_opts[] = {
 {"history", 'h',  "NUM",          N_("History"),     ARGUMENT_TYPE_VALUE, &arg_h},
 {"new",     'n',  "ON/OFF",       N_("New File"),    ARGUMENT_TYPE_BOOL,  &arg_n},
+{"profile", 'p',  "ON/OFF",       N_("Profile"),     ARGUMENT_TYPE_BOOL,  &arg_p},
 {"size",    's',  "WIDTH,HEIGHT", N_("Window Size"), ARGUMENT_TYPE_ARRAY, &arg_s},
 {"second",  'x',  "ON/OFF",       N_("Open Window"), ARGUMENT_TYPE_BOOL,  &arg_x},
 {"version", 'v',  NULL,           N_("About"),       ARGUMENT_TYPE_EXIT,  &arg_v},
@@ -420,6 +357,9 @@ main (int   argc,
     /* ja:新規ファイル */
     arg_n.flag = newfile;
     arg_n.def = FALSE;
+    /* ja:レジストリ */
+    arg_p.flag = TRUE;
+    arg_p.def = TRUE;
     /* ja:ウインドウサイズ */
     array[0] = def_width;
     array[1] = def_height;
@@ -441,6 +381,9 @@ MMAID_COPYRIGHT_NOTICE"\n"
     arg_q.str = MMAID_COPYRIGHT_NOTICE"\n\n";
     arg_files = arg_analyse (argv, arg_opts);
     orz_history_set_num (ORZ_HISTORY (history), arg_h.val);
+    newfile = arg_n.flag;
+    if (!arg_p.flag)
+      user_config = NULL;
     def_width = array[0];
     def_height = array[1];
     second = arg_x.flag;
@@ -502,8 +445,16 @@ MMAID_COPYRIGHT_NOTICE"\n"
   accel_group = gtk_accel_group_new ();
   misc_create_menu (menu_entries, accel_group);
   /* ja:ファイルの履歴 */
+  count = 0;
+  for (i = 0; menu_entries[i].path; i++)
+#if GLIB_CHECK_VERSION(2,2,0)
+    if (g_str_has_prefix (menu_entries[i].path, "/file/"))
+#else /* not GLIB_CHECK_VERSION(2,2,0) */
+    if (g_strncmp (menu_entries[i].path, "/file/", 6) == 0)
+#endif /* not GLIB_CHECK_VERSION(2,2,0) */
+      count++;
   orz_history_set_menu (ORZ_HISTORY (history),
-                        misc_find_menu (menu_entries, "/file/"), MENUFILE - 2);
+                        misc_find_menu (menu_entries, "/file/"), count - 3);
   g_signal_connect (ORZ_HISTORY (history), "activate",
                             G_CALLBACK (signal_activate_menu_history), NULL);
   /* ja:ウインドウメニュー */
@@ -555,35 +506,39 @@ MMAID_COPYRIGHT_NOTICE"\n"
   count = 0;
   for (i = 1; i < argc; i++)
     if (arg_files[i])
-      switch (file_is_type (argv[i]))
-        {
-          case FILE_KIND_ERROR:
-          case FILE_KIND_DB:
-          case FILE_KIND_LIST:
-            mmaid = file_open_edit (argv[i]);
-            count++;
-            break;
-          case FILE_KIND_UNKNOWN:
-          case FILE_KIND_TAG:
-            {
-              ID3Tag *id3tag;
+      {
+        MmaidWindow *mmaid = NULL;
 
-              id3tag = file_open_id3tag (argv[i], charset_list);
-              if (id3tag && (!mmaid
+        switch (file_is_type (argv[i]))
+          {
+            case FILE_KIND_ERROR:
+            case FILE_KIND_DB:
+            case FILE_KIND_LIST:
+              mmaid = file_open_edit (argv[i]);
+              count++;
+              break;
+            case FILE_KIND_UNKNOWN:
+            case FILE_KIND_TAG:
+              {
+                ID3Tag *id3tag;
+
+                id3tag = file_open_id3tag (argv[i], charset_list);
+                if (id3tag && (!mmaid
                     || mm_list_search (MM_LIST (mmaid->list),
                                         MM_LIST_FILE, id3tag->file, -1) < 0))
-                {
-                  if (!mmaid)
-                    {
-                      mmaid = file_open_edit (NULL);
-                      count++;
-                    }
-                  mm_list_append (MM_LIST (mmaid->list), id3tag);
-                  orz_mdi_set_edited (ORZ_MDI (mdi),
+                  {
+                    if (!mmaid)
+                      {
+                        mmaid = file_open_edit (NULL);
+                        count++;
+                      }
+                    mm_list_append (MM_LIST (mmaid->list), id3tag);
+                    orz_mdi_set_edited (ORZ_MDI (mdi),
                     orz_mdi_get_page_from_data (ORZ_MDI (mdi), mmaid), TRUE);
-                }
-            }
-        }
+                  }
+              }
+          }
+      }
   g_free (arg_files);
   if (newfile && count <= 0)
     file_open_edit (NULL);
@@ -591,95 +546,119 @@ MMAID_COPYRIGHT_NOTICE"\n"
   gtk_main();
 
   /* ja:レジストリに書き込む */
-  key_file = g_key_file_new ();
-  g_key_file_load_from_file (key_file, user_config, G_KEY_FILE_NONE, NULL);
-  g_key_file_set_integer (key_file, "mmaid", "license", license);
-  g_key_file_set_integer (key_file, "mmaid", "history",
-                                orz_history_get_num (ORZ_HISTORY (history)));
-  g_key_file_set_boolean (key_file, "mmaid", "newfile", newfile);
-  g_key_file_set_boolean (key_file, "mmaid", "second", second);
-  g_key_file_set_integer (key_file, "mmaid", "def_width", def_width);
-  g_key_file_set_integer (key_file, "mmaid", "def_height", def_height);
-  g_key_file_set_integer (key_file, "mmaid", "n_pos", n_pos);
-  g_key_file_set_boolean (key_file, "mmaid", "def_hold", def_hold);
-  g_key_file_set_integer (key_file, "mmaid", "def_gnum", def_gnum);
-  g_key_file_set_integer (key_file, "mmaid", "playlist", playlist);
-  g_key_file_set_string (key_file, "mmaid", "cmd_play", cmd_play);
-  g_key_file_set_string (key_file, "mmaid", "cmd_stop", cmd_stop);
-  g_key_file_set_string (key_file, "mmaid", "charset_list", charset_list);
-  if (open_path)
+  if (user_config)
     {
-      utf8str = g_filename_to_utf8 (open_path, -1, NULL, NULL, NULL);
-      if (utf8str)
-        {
-          g_key_file_set_string (key_file, "mmaid", "openpath", utf8str);
-          g_free (utf8str);
-        }
-    }
-  if (save_path)
-    {
-      utf8str = g_filename_to_utf8 (save_path, -1, NULL, NULL, NULL);
-      if (utf8str)
-        {
-          g_key_file_set_string (key_file, "mmaid", "savepath", utf8str);
-          g_free (utf8str);
-        }
-    }
-  /* ja:ファイルの履歴 */
-  glist = orz_history_get_file (ORZ_HISTORY (history));
-  for (i = 0; i < g_list_length (glist); i++)
-    {
-      key = g_strdup_printf ("history%d", i);
-      utf8str = g_filename_to_utf8 (g_list_nth_data (glist, i), -1,
-                                                            NULL, NULL, NULL);
-      if (utf8str)
-        {
-          g_key_file_set_string (key_file, "mmaid", key, utf8str);
-          g_free (utf8str);
-        }
-      g_free (key);
-    }
-  g_list_free (glist);
-  while (i <= ORZ_HISTORY_NUM_MAX)
-    {
-      key = g_strdup_printf ("history%d", i++);
-      g_key_file_remove_key (key_file, "mmaid", key, NULL);
-      g_free (key);
-    }
-  /* ja:ジャンル */
-  for (i = 0; i < genres; i++)
-    {
-      group_name = g_strdup_printf ("genre%03d", i);
-      for (j = 0; j < 256; j++)
-        {
-          key = g_strdup_printf ("genre%03d", j);
+      gsize length;
+      gchar *data;
 
-          if (genre[i][j])
-            g_key_file_set_string (key_file, group_name, key, genre[i][j]);
-          else
-            g_key_file_remove_key (key_file, group_name, key, NULL);
+      key_file = g_key_file_new ();
+      g_key_file_load_from_file (key_file, user_config, G_KEY_FILE_NONE, NULL);
+      g_key_file_set_integer (key_file, "license", "hash", license);
+      g_key_file_set_integer (key_file, "pref", "history",
+                                orz_history_get_num (ORZ_HISTORY (history)));
+      g_key_file_set_boolean (key_file, "pref", "newfile", newfile);
+      g_key_file_set_boolean (key_file, "pref", "second", second);
+      g_key_file_set_integer (key_file, "pref", "width", def_width);
+      g_key_file_set_integer (key_file, "pref", "height", def_height);
+      g_key_file_set_integer (key_file, "pref", "n_pos", n_pos);
+      g_key_file_set_boolean (key_file, "mmaid", "def_hold", def_hold);
+      g_key_file_set_integer (key_file, "mmaid", "def_gnum", def_gnum);
+      g_key_file_set_integer (key_file, "mmaid", "playlist", playlist);
+      g_key_file_set_string (key_file, "mmaid", "charset_list", charset_list);
+      if (cmd_play)
+        g_key_file_set_string (key_file, "mmaid", "cmd_play", cmd_play);
+      else
+        g_key_file_remove_key (key_file, "mmaid", "cmd_play", NULL);
+      if (cmd_stop)
+        g_key_file_set_string (key_file, "mmaid", "cmd_stop", cmd_stop);
+      else
+        g_key_file_remove_key (key_file, "mmaid", "cmd_stop", NULL);
+      if (open_path)
+        {
+          gchar *utf8str;
+
+          utf8str = g_filename_to_utf8 (open_path, -1, NULL, NULL, NULL);
+          if (utf8str)
+            {
+              g_key_file_set_string (key_file, "mmaid", "openpath", utf8str);
+              g_free (utf8str);
+            }
+        }
+      if (save_path)
+        {
+          gchar *utf8str;
+
+          utf8str = g_filename_to_utf8 (save_path, -1, NULL, NULL, NULL);
+          if (utf8str)
+            {
+              g_key_file_set_string (key_file, "mmaid", "savepath", utf8str);
+              g_free (utf8str);
+            }
+        }
+      /* ja:ファイルの履歴 */
+      glist = orz_history_get_file (ORZ_HISTORY (history));
+      for (i = 0; i < g_list_length (glist); i++)
+        {
+          gchar *key, *utf8str;
+
+          key = g_strdup_printf ("file%d", i);
+          utf8str = g_filename_to_utf8 (g_list_nth_data (glist, i), -1,
+                                                            NULL, NULL, NULL);
+          if (utf8str)
+            {
+              g_key_file_set_string (key_file, "history", key, utf8str);
+              g_free (utf8str);
+            }
           g_free (key);
         }
-      g_free (group_name);
-    }
-  while (i < 256)
-    {
-      group_name = g_strdup_printf ("genre%03d", i++);
-      g_key_file_remove_group (key_file, group_name, NULL);
-      g_free (group_name);
-    }
-  data = g_key_file_to_data (key_file, &length, NULL);
-  g_key_file_free (key_file);
-  if (data)
-    {
-      fileio_save (user_config, data, length);
-      g_free (data);
+      g_list_free (glist);
+      while (i <= ORZ_HISTORY_NUM_MAX)
+        {
+          gchar *key;
+
+          key = g_strdup_printf ("file%d", i++);
+          g_key_file_remove_key (key_file, "history", key, NULL);
+          g_free (key);
+        }
+      /* ja:ジャンル */
+      for (i = 0; i < genres; i++)
+        {
+          gchar *group_name;
+
+          group_name = g_strdup_printf ("genre%03d", i);
+          for (j = 0; j < 256; j++)
+            {
+              gchar *key;
+
+              key = g_strdup_printf ("genre%03d", j);
+              if (genre[i][j])
+                g_key_file_set_string (key_file, group_name, key, genre[i][j]);
+              else
+                g_key_file_remove_key (key_file, group_name, key, NULL);
+              g_free (key);
+            }
+          g_free (group_name);
+        }
+      while (i < 256)
+        {
+          gchar *group_name;
+
+          group_name = g_strdup_printf ("genre%03d", i++);
+          g_key_file_remove_group (key_file, group_name, NULL);
+          g_free (group_name);
+        }
+      data = g_key_file_to_data (key_file, &length, NULL);
+      g_key_file_free (key_file);
+      if (data)
+        {
+          fileio_save (user_config, data, length);
+          g_free (data);
+        }
     }
 
   for (i = 0; i < genres; i++)
     for (j = 0; j < 256; j++)
       g_free (genre[i][j]);
-  g_free (user_config);
   g_free (clipboard_id3tag);
   g_free (open_path);
   g_free (save_path);
