@@ -75,6 +75,18 @@ reload_dialog (MmaidWindow *mmaid,
   g_signal_connect (G_OBJECT (dialog), "key-press-event",
                                     G_CALLBACK (misc_dialog_key_press), NULL);
   /* ja:ボタン */
+#ifdef G_OS_WIN32
+  if (text)
+    gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+                            GTK_STOCK_YES, GTK_RESPONSE_ACCEPT,
+                            GTK_STOCK_NO, GTK_RESPONSE_REJECT,
+                            NULL);
+  else
+    gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+                            GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+                            NULL);
+#else /* not G_OS_WIN32 */
   if (text)
     gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                             GTK_STOCK_NO, GTK_RESPONSE_REJECT,
@@ -85,6 +97,7 @@ reload_dialog (MmaidWindow *mmaid,
                             GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
                             GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
                             NULL);
+#endif /* not G_OS_WIN32 */
   /* ja:文字符号化方式 */
   charset = orz_charset_new ();
   orz_charset_add_candidate (ORZ_CHARSET (charset), charset_list);
